package org.mai.dep810.reflection;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.sql.*;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

public class EntityDaoImpl implements EntityDao {

    private Connection connection;

    public EntityDaoImpl(Connection connection) {
        this.connection = connection;
    }

    private String getTableName(Class type) {
        String tableName = null;

        for(Annotation a : type.getAnnotations()){
            if(a.annotationType().getCanonicalName() == "org.mai.dep810.reflection.Table"){
                Table t = (Table)(a);
                tableName = t.name();
                break;
            }
        }

        return tableName;
    }

    private Map<String, Field> getMapping(Class type) {
        Map<String, Field> columnMapping = new HashMap<>();

        for(Field field : type.getDeclaredFields()){
            Column column = field.getAnnotation(Column.class);
            field.setAccessible(true);
            columnMapping.put(column.name(), field);
        }
        return columnMapping;
    }

    public <T> List<T> getEntities(Class<T> type) {

        String tableName = getTableName(type);

        Map<String, Field> columnMapping = getMapping(type);

        String sql = "select * from " + tableName;

        List<T> result = new ArrayList<>();
        try (
                Statement stmt = connection.createStatement();
                ResultSet rs = stmt.executeQuery(sql);)
        {
            while (rs.next()) {
                T entity = type.newInstance();
                for (Map.Entry<String, Field> entry : columnMapping.entrySet()) {
                    entry.getValue().set(entity, rs.getObject(entry.getKey()));
                }
                result.add(entity);
            }

        } catch (SQLException | InstantiationException | IllegalAccessException e1) {
            e1.printStackTrace();
        }


        return result;
    }

    public <T> boolean saveEntity(T entity) throws IllegalAccessException {

        boolean result = false;
        Field[] fields = entity.getClass().getDeclaredFields();

        String tableName = getTableName(entity.getClass());
        String columns = Arrays
                .stream(fields)
                .map(f -> f.getAnnotation(Column.class).name())
                .collect(Collectors.joining(","));
        String wildcards = IntStream.range(0, fields.length).mapToObj(i -> "?").collect(Collectors.joining(","));
        String sql = "insert into " + tableName + " (" + columns + ") values(" + wildcards + ")";

        try{
            PreparedStatement stmt = connection.prepareStatement(sql);

            for(int i = 0; i < fields.length; i++) {
                fields[i].setAccessible(true);
                stmt.setObject(i+1, fields[i].get(entity));
            }

            result = stmt.execute();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return result;
    }

}
