package org.mai.dep810.reflection;

import org.apache.log4j.Logger;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Arrays;
import java.util.List;

public class WorkWithAnnotation {

    private static Logger log = Logger.getLogger(WorkWithAnnotation.class);

    public static void main(String[] args) throws SQLException {
        Connection connection = DriverManager.getConnection("jdbc:derby:memory:database;create=true");

        createTables(connection);

        EntityDao dao = new EntityDaoImpl(connection);

        //сохранение студентов в БД
        Student[] students = new Student[] {
                new Student(1, "Ivanov"),
                new Student(2, "Petrov"),
                new Student(3, "Sidorov"),
                new Student(4, "Fedorov")
        };

        Arrays.stream(students).forEach(student -> {
            try {
                dao.saveEntity(student);
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        });

        List<Student> studentEntities = dao.getEntities(Student.class);
        log.info(studentEntities);

        //сохранение книг в БД
        Book[] books = new Book[] {
                new Book(1, "The first book", 1),
                new Book(2, "The second book", 1),
                new Book(3, "The third book", null),
        };
        Arrays.stream(books).forEach(book -> {
            try {
                dao.saveEntity(book);
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        });
        List<Book> bookEntities = dao.getEntities(Book.class);
        log.info(bookEntities);

    }

    static void createTables(Connection connection) {
        try (Statement stmt = connection.createStatement();) {
            String tableSql = "CREATE TABLE STUDENTS(\n" +
                    "    student_id int primary key,\n" +
                    "    student_name varchar(255)\n" +
                    ")";
            stmt.execute(tableSql);

            tableSql = "CREATE TABLE BOOKS(\n" +
                    "    book_id int primary key,\n" +
                    "    book_title varchar(255),\n" +
                    "    student_id int,\n" +
                    "    foreign key(student_id) references STUDENTS(student_id)\n" +
                    ")";
            stmt.execute(tableSql);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }
}
