package org.mai.dep810.reflection;

import org.apache.log4j.Logger;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

public class WorkWithJDKProxy {

    private static Logger log = Logger.getLogger(WorkWithJDKProxy.class);

    public static void main(String[] args) throws IllegalAccessException, SQLException {

        Connection connection = DriverManager.getConnection("jdbc:derby:memory:database;create=true");
        createTables(connection);

        EntityDao dao = (EntityDao) Proxy.newProxyInstance(EntityDao.class.getClassLoader(), new Class[]{EntityDao.class}, new LogProxy(new EntityDaoImpl(connection)));

        dao.saveEntity(new Student(1, "Petrov"));

    }

    static void createTables(Connection connection) {
        try (Statement stmt = connection.createStatement();) {
            String tableSql = "CREATE TABLE STUDENTS(\n" +
                    "    student_id int primary key,\n" +
                    "    student_name varchar(255)\n" +
                    ")";
            stmt.execute(tableSql);

            tableSql = "CREATE TABLE BOOKS(\n" +
                    "    book_id int primary key,\n" +
                    "    book_title varchar(255),\n" +
                    "    student_id int,\n" +
                    "    foreign key(student_id) references STUDENTS(student_id)\n" +
                    ")";
            stmt.execute(tableSql);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }
}

class LogProxy implements InvocationHandler {

    private final Logger log = Logger.getLogger(this.getClass());

    private Object original;

    public LogProxy(Object original) {
        this.original = original;
    }

    @Override
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {

        log.info("calling method "+method.getName());
        Object result = method.invoke(original, args);
        log.info("End of calling method "+method.getName());
        return result;

    }
}