package org.mai.dep810.reflection;

import org.apache.log4j.Logger;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.*;
import java.util.stream.Collectors;

public class WorkWithReflection {

    private static Logger log = Logger.getLogger(WorkWithReflection.class);


    public static void main(String[] args) throws IllegalAccessException, InstantiationException, ClassNotFoundException, NoSuchMethodException, InvocationTargetException {
        log.info("Start");

        Class studentType = Class.forName("org.mai.dep810.reflection.Student");
        Object ivanovObj = studentType
                .getDeclaredConstructor(Integer.class, String.class)
                .newInstance(2, "Ivanov");
        Student ivanov = (Student)ivanovObj;
        log.info(ivanov);

        Student petrov = new Student(1, "Petrov");

        //создадим книгу
        Class bookType = Class.forName("org.mai.dep810.reflection.Book");
        Object bookObj = bookType
                .getDeclaredConstructor(Integer.class, String.class, Integer.class)
                .newInstance(1, "The Book", 1);
        Book book = (Book)bookObj;
        log.info(book);

        //вызов публичного метода
        Method methodPublic = bookType.getMethod("updateBorrowerPublic", Student.class);
        methodPublic.invoke(bookObj, ivanov);

        //вызов приватного метода
        Method methodPrivate = bookType.getDeclaredMethod("updateBorrowerPrivate", Student.class);
        methodPrivate.setAccessible(true);
        methodPrivate.invoke(bookObj, petrov);

        //распечатает список полей
        printFields(book);

        //распачатает список методов
        printMethods(petrov);

        //конвертация сущности в карту
        Map<String, Object> map = convertToMap(book);
        log.info(map);
        //конвертация map в сущность
        Book anotherBook = convertToEntity(map, Book.class);
        log.info(anotherBook);

        //конвертация сущности в json
        log.info(convertToJson(ivanov));

        log.info("Finish");
    }

    static void printFields(Object o) throws IllegalAccessException {
        Class c = o.getClass();
        for(Field field : c.getDeclaredFields()){
            field.setAccessible(true);

            log.info(""+
                    Modifier.toString(field.getModifiers()) + " " +
                    field.getType().getCanonicalName() + " " +
                    field.getName() + " = " +
                    field.get(o)
            );
        }
    }

    static void printMethods(Object o) {
        Class c = o.getClass();

        for(Method method : c.getMethods()) {

            String parameterList = Arrays
                    .stream(method.getParameters())
                    .map(p -> p.getType().getCanonicalName() + " " + p.getName())
                    .collect(Collectors.joining(","));

            log.info("" +
                    Modifier.toString(method.getModifiers()) + " " +
                    method.getReturnType().getCanonicalName() + " " +
                    method.getName() + "(" + parameterList + ")"
            );
        }
    }

    static Map<String, Object> convertToMap(Object o) throws IllegalAccessException {
        Map<String, Object> result = new HashMap<>();
        Class c = o.getClass();

        for(Field field : c.getDeclaredFields()) {
            field.setAccessible(true);
            result.put(field.getName(), field.get(o));
        }

        return result;
    }

    static <T> T convertToEntity(Map<String, Object> map, Class<T> type) throws IllegalAccessException, InstantiationException {

        T result = type.newInstance();

        Arrays.stream(type.getDeclaredFields()).forEach(field -> {
            try {
                field.setAccessible(true);
                field.set(result, map.get(field.getName()));
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        });

        return result;
    }

    static String convertToJson(Object o) throws IllegalAccessException {
        Map<String, Object> fields = convertToMap(o);

        return "{"+fields.entrySet().stream().map(e -> "\"" + e.getKey() + "\": \"" + e.getValue() + "\"").collect(Collectors.joining(", "))+"}";
    }


}
